(function (root, factory) {
  if (typeof define === 'function' && define.amd) {
    define([], factory);
  } else if (typeof exports === 'object' && typeof module !== "undefined") {
    module.exports = factory();
  } else {
    root.checkboxTree = factory();
  }
}(this, function () {
    var checkboxTree = function (node, settings) {
      var defaultLevel = {
        children: 'children',
        checkedProperty: 'flag',
        name: 'name',
        id: 'id',
        parentId: 'parent_id'
      }
      var level = this.extend(defaultLevel, settings.level);
      this.node = node;
      this.data = settings.data;
      this.childrenSign = level.children;
      this.checkedProperty = level.checkedProperty;
      this.nameSign = level.name;
      this.idSign = level.id;
      this.parentIdSign = level.parentId;
      this.changeHandler =  settings.changeHandler;
      this.created = settings.created;
      this.dataMap = {};
      this.selectedData = [];
      this.getSelectedNodes = this._getSelectedNodes
      this.init();
    }
    checkboxTree.prototype = {
      extend: function (sub, supe) {
        Object.keys(sub).forEach(function (value) {
          if (supe[value]) {
            sub[value] = supe[value]
          }
        })
        return sub
      },
      init: function () {
        var checkboxTree = this.$createElement('div', {className: 'checkbox-tree'})
        checkboxTree.appendChild(this.getTreeDom(this.data))
        this.node.appendChild(checkboxTree)

        var treeLayout = this.node.querySelector('.checkbox-tree')
        treeLayout.addEventListener('click', this.clickHandler.bind(this))
        if (this.created) this.created()
      },
      getTreeDom: function (nodeData) {
        var fregment = document.createDocumentFragment()
        var empty = true
        nodeData.forEach(function (value) {
          this.dataMap[value['id']] = value
          if (value[this.childrenSign] && value[this.childrenSign].length) {
            var indeterminate = false
            if (value[this.checkedProperty]) {
              indeterminate = this.getIndeterminate(value.children, false)
            } else {
              indeterminate = this.getIndeterminate(value.children, true)
            }
            var treeNode = this.getTreeNode()
            treeNode.appendChild(this.getTreeNodeContent(value[this.nameSign], value[this.idSign], value[this.checkedProperty], true, indeterminate))
            treeNode.appendChild(this.getTreeNodeChild(this.getTreeDom(value[this.childrenSign])))
            fregment.appendChild(treeNode)
          } else {
            var treeNode = this.getTreeNode()
            treeNode.appendChild(this.getTreeNodeContent(value[this.nameSign], value[this.idSign], value[this.checkedProperty], false, indeterminate))
            fregment.appendChild(treeNode)
          }
        }, this)
        return fregment
      },
      getTreeNodeContent: function (text, id, checked, empty, indeterminate) {
        var emptySign = empty ? '' : 'empty'
        var node = this.$createElement('div', {className: 'tree-node-content'})
        node.appendChild(this.$createElement('span', {className: 'tree-icon ' + emptySign}))
        node.appendChild(this.$createElement('input', {type: 'checkbox', name: id, checked: checked, indeterminate: indeterminate}))
        node.appendChild(this.$createElement('span', {className: 'tree-text', innerHTML: text}))
        return node
      },
      getTreeNode: function (content) {
        var node = this.$createElement('div', {className: 'tree-node'})
        return node
      },
      getIndeterminate: function (children, boo) {
        return children.some(function (child) {
          if (child[this.checkedProperty] === boo) {
            return true
          } else {
            if (child[this.childrenSign] && child[this.childrenSign].length) {
              return this.getIndeterminate(child[this.childrenSign], boo)
            } else {
              return false
            }
          }
        }, this)
      },
      getTreeNodeChild: function (content) {
        var node = this.$createElement('div', {className: 'tree-node-child'})
        node.appendChild(content)
        return node
      },
      clickHandler: function (event) {
        var target = event.target
        if (target.tagName === 'SPAN') {
          var clickChildrenDom = target.parentNode.nextSibling
          if (!clickChildrenDom) return
          if (clickChildrenDom.style.display === 'block') {
            clickChildrenDom.style.display = 'none'
            target.classList.remove('expened')
          } else {
            clickChildrenDom.style.display = 'block'
            target.classList.add('expened')
          }
        } else if (target.tagName === 'INPUT') {
          this.checkboxClickHandler(target)
        }
      },
      handleParentChange: function (parentDom, checked, childIndeterminate) {
        if (!parentDom.previousSibling) { return }
        var parentInput = parentDom.previousSibling.querySelector('input')
        var indeterminate = false
        if (checked) {
          for (var i = 0; i < parentDom.children.length; i++) {
            if (!parentDom.children[i].children[0].querySelectorAll('input')[0].checked) {
              indeterminate = true
              break
            }
          }
          parentInput.checked = true
          parentInput.indeterminate = childIndeterminate || indeterminate
        } else {
          for (var i = 0; i < parentDom.children.length; i++) {
            if (parentDom.children[i].children[0].querySelectorAll('input')[0].checked) {
              indeterminate = true
              break
            }
          }
          parentInput.checked = indeterminate
          parentInput.indeterminate = childIndeterminate || indeterminate
        }
        if (parentDom.parentNode.parentNode.className === 'tree-node-child') {
          this.handleParentChange(parentDom.parentNode.parentNode, checked, indeterminate)
        }
      },
      $createElement: function (tagName, attrs) {
        var node = document.createElement(tagName)
        for (var attr in attrs) {
          if (attrs.hasOwnProperty(attr)) {
            node[attr]= attrs[attr]
          }
        }
        return node
      },
      _getSelectedNodes: function () {
        var input = this.node.querySelectorAll('input');
        var selectedData = []
        input.forEach(function (value) {
          if (value.checked) {
            selectedData.push(this.dataMap[value.name])
          }
        }, this)
        return selectedData
      },
      checkboxClickHandler: function (target) {
        var clickChildrenDom = target.parentNode.nextSibling
        var parentDom = target.parentNode.parentNode.parentNode
        var indeterminate = false
        if (target.checked === true) {
          if (clickChildrenDom) {  
            clickChildrenDom.querySelectorAll('input').forEach(function (value) {
              value.checked = true
            })
          }
          this.handleParentChange(parentDom, true, false)
        } else {
          if (clickChildrenDom) { 
            clickChildrenDom.querySelectorAll('input').forEach(function (value) {
              value.checked = false
            })
          }
          this.handleParentChange(parentDom, false, false)
        }
      }
    }
    return checkboxTree
}));
